--****************************************************************************** 
--*COPYRIGHT: BENJAMIN GRYDEHOEJ - WWW.BG-ELEKTRONIK.DK - 2006 - FPGA SPARTAN-3* 
--******************************************************************************
--  Author:          Benjamin Grydehoej
--  Create the:      11th November, 2005
--  Last update the: 14th April, 2006
--  File:            EMBEDDED.VHD
--  Target Hardware: Xilinx Spartan3 - XC3S200
--  Tool chain:      Xilinx - Project Navicator 6.3.03i
--  Version:         1.0.D
--
--  DESCRIPTION:
--  ============
--  This VHDL project is the main code; there connect all the small programs 
--  functions in one program. With map the other programs to this code and 
--  links them together.
--  The functions of this program is at read code from the Program Rom and run 
--  then in the processor core and write and read to the SFR register or I/O ports. 
--
--------------------------------------------------------------------------
--------------------------------------------------------------------------
-- Standard IEEE libraries
--------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--------------------------------------------------------------------------
entity embedded is
    Port (           tx : out std_logic;
                     rx : in  std_logic;
                  reset : in  std_logic;
                    clk : in  std_logic;
    			  PORT_0 : inout std_logic_vector(7 downto 0);
    			  PORT_1 : inout std_logic_vector(7 downto 0);
    			  PORT_2 : out   std_logic_vector(15 downto 0);
    			  PORT_3 : inout std_logic_vector(7 downto 0);
			  PORT_4 : inout std_logic_vector(7 downto 0);
			  PORT_5 : inout std_logic_vector(7 downto 0));				
end embedded;
--------------------------------------------------------------------------
-- Start of embedded achitecture
--------------------------------------------------------------------------
architecture connectivity of embedded is
--------------------------------------------------------------------------
-- declaration of KCPSM3
--------------------------------------------------------------------------
  component kcpsm3
    Port (      address : out std_logic_vector(9 downto 0);
            instruction : in  std_logic_vector(17 downto 0);
                port_id : out std_logic_vector(7 downto 0);
           write_strobe : out std_logic;
               out_port : out std_logic_vector(7 downto 0);
            read_strobe : out std_logic;
                in_port : in  std_logic_vector(7 downto 0);
              interrupt : in  std_logic;
          interrupt_ack : out std_logic;
                  reset : in  std_logic;
                    clk : in  std_logic);
    end component;
--------------------------------------------------------------------------
-- declaration of program ROM
--------------------------------------------------------------------------
  component prog_rom 
    Port (      address : in  std_logic_vector(9 downto 0);
            instruction : out std_logic_vector(17 downto 0);
                    clk : in  std_logic);
    end component;
--------------------------------------------------------------------------
-- declaration of I/O Ports
--------------------------------------------------------------------------
  component in_out_ports
    Port (          CLK_io : in  	std_logic;
			   Reset_io : in  	std_logic;
    		           WE_io : in  	std_logic;
			      RE_io : in  	std_logic;
   	                ID_io : in  	std_logic_vector(7 	downto 0);
			 Data_in_io : in  	std_logic_vector(7 	downto 0);
			Data_out_io : out 	std_logic_vector(7 	downto 0);
		    Interrupt_io : out 	std_logic;
		Interrupt_ack_io : in 	std_logic;
		 rx_data_present : in	std_logic;
    			  Port_0_io : inout std_logic_vector(7 	downto 0);
  			  Port_1_io : inout std_logic_vector(7 	downto 0);
  			  Port_2_io : out 	std_logic_vector(15 downto 0);
  			  Port_3_io : inout std_logic_vector(7 	downto 0);
			  Port_4_io : inout std_logic_vector(7 	downto 0);
			  Port_5_io : inout std_logic_vector(7 	downto 0);
			   TBS_uart : out  	std_logic_vector(15 downto 0);
			   BDP_uart : in  	std_logic;
			   RBH_uart : in  	std_logic;
			   RBF_uart : in  	std_logic;
			   TBH_uart : in  	std_logic;
			   TBF_uart : in  	std_logic;
	        write_to_uart : out 	std_logic;
		    data_to_uart : out 	std_logic_vector(7 	downto 0);
		  read_from_uart : out 	std_logic;
		  data_from_uart : in  	std_logic_vector(7 	downto 0);
			  TR0_timer : inout std_logic;
			  TF0_timer : in  std_logic;
			  TC0_timer : out std_logic_vector(7 downto 0);
			  TR1_timer : inout std_logic;
			  TF1_timer : in  std_logic;
			  TC1_timer : out std_logic_vector(15 downto 0)
);			  	 
    end component;
--------------------------------------------------------------------------
-- declaration of UART transmitter with integral 16 byte FIFO buffer
--------------------------------------------------------------------------
  component uart_tx
    Port (            data_in : in std_logic_vector(7 downto 0);
                 write_buffer : in std_logic;
                 reset_buffer : in std_logic;
                 en_16_x_baud : in std_logic;
                   serial_out : out std_logic;
                  buffer_full : out std_logic;
             buffer_half_full : out std_logic;
                          clk : in std_logic);
    end component;
--------------------------------------------------------------------------
-- declaration of UART Receiver with integral 16 byte FIFO buffer
--------------------------------------------------------------------------
  component uart_rx
    Port (            serial_in : in std_logic;
                       data_out : out std_logic_vector(7 downto 0);
                    read_buffer : in std_logic;
                   reset_buffer : in std_logic;
                   en_16_x_baud : in std_logic;
            buffer_data_present : out std_logic;
                    buffer_full : out std_logic;
               buffer_half_full : out std_logic;
                            clk : in std_logic);
  
  end component;
--------------------------------------------------------------------------
-- Signals used to connect KCPSM3 to program ROM
--------------------------------------------------------------------------
signal       address : std_logic_vector(9 downto 0);
signal   instruction : std_logic_vector(17 downto 0);
--------------------------------------------------------------------------
-- Signals used to connect KCPSM3 to I/O Ports
--------------------------------------------------------------------------
signal        port_id : std_logic_vector(7 downto 0);
signal       out_port : std_logic_vector(7 downto 0);
signal        in_port : std_logic_vector(7 downto 0);
signal   write_strobe : std_logic;
signal    read_strobe : std_logic;
signal      interrupt : std_logic;
signal  interrupt_ack : std_logic;
signal   data_to_uart : std_logic_vector(7 downto 0);
signal data_from_uart : std_logic_vector(7 downto 0);
--------------------------------------------------------------------------
-- Signals for UART connections
--------------------------------------------------------------------------
signal           baud_count : std_logic_vector(15 downto 0) := X"0000";
signal	 	    TBS_uart : std_logic_vector(15 downto 0); 
signal         en_16_x_baud : std_logic;
signal        write_to_uart : std_logic;
signal              tx_full : std_logic;
signal         tx_half_full : std_logic;
signal       read_from_uart : std_logic;
signal      rx_data_present : std_logic;
signal              rx_full : std_logic;
signal         rx_half_full : std_logic;
--------------------------------------------------------------------------
-- Timer0 control
--------------------------------------------------------------------------
signal          	 Timer0 : std_logic_vector(7 downto 0) := X"00";
signal			    TR0 : std_logic;
signal			    TF0 : std_logic;
signal			    TC0 : std_logic_vector(7 downto 0); 
--------------------------------------------------------------------------
-- Timer1 control
--------------------------------------------------------------------------
signal          	 Timer1 : std_logic_vector(15 downto 0) := X"0000";
signal			    TR1 : std_logic;
signal			    TF1 : std_logic;
signal			    TC1 : std_logic_vector(15 downto 0); 
--------------------------------------------------------------------------
-- Start of test circuit description
--------------------------------------------------------------------------
begin
  processor: kcpsm3
    port map(      address => address,
               instruction => instruction,
                   port_id => port_id,
              write_strobe => write_strobe,
                  out_port => out_port,
               read_strobe => read_strobe,
                   in_port => in_port,
                 interrupt => interrupt,
             interrupt_ack => interrupt_ack,
                     reset => reset,
                       clk => clk);

  program: prog_rom
    port map(      address => address,
               instruction => instruction,
                       clk => clk);

  in_out_port: in_out_ports
    port map(          CLK_io => clk,
    				 Reset_io => reset,
    		    		    WE_io => write_strobe,
				    RE_io => read_strobe,
    	              	    ID_io => port_id,
			    Data_in_io => out_port,
			   Data_out_io => in_port,
		       Interrupt_io => interrupt,
		   Interrupt_ack_io => interrupt_ack,
		    rx_data_present => rx_data_present,
    			     Port_0_io => port_0,
    			     Port_1_io => port_1,
			     Port_2_io => port_2,
				Port_3_io => port_3,
    			     Port_4_io => port_4,
			     Port_5_io => port_5,
				 TBS_uart	=> TBS_uart,
		           BDP_uart => rx_data_present,
			      RBH_uart => rx_half_full,
			      RBF_uart => rx_full,
			      TBH_uart => tx_half_full,
			      TBF_uart => tx_full,
			 write_to_uart => write_to_uart,
			  data_to_uart => data_to_uart,
			read_from_uart => read_from_uart,
			data_from_uart => data_from_uart,
				TR0_timer => TR0,
				TF0_timer => TF0,
				TC0_timer => TC0,
				TR1_timer => TR1,
				TF1_timer => TF1,
				TC1_timer => TC1);
------------------------------------------------------------------------- 
-- Connect the 8-bit, 1 stop-bit, no parity transmit and receive macros.
-- Each contains an embedded 16-byte FIFO buffer.
-------------------------------------------------------------------------

  transmit: uart_tx 
  port map (            data_in => data_to_uart, 
                   write_buffer => write_to_uart,
                   reset_buffer => reset,
                   en_16_x_baud => en_16_x_baud,
                     serial_out => tx,
                    buffer_full => tx_full,
               buffer_half_full => tx_half_full,
                            clk => clk );

  receive: uart_rx
  port map (            serial_in => rx,
                         data_out => data_from_uart,
                      read_buffer => read_from_uart,
                     reset_buffer => '0',
                     en_16_x_baud => en_16_x_baud,
              buffer_data_present => rx_data_present,
                      buffer_full => rx_full,
                 buffer_half_full => rx_half_full,
                              clk => clk );  
  
-----------------------------------------------------------------------------
-- BAUD RAGE TIMER:
-- This function sets the baud rate to the value for the UART communications, 
-- after reset is the value default integer 326 or HEX value 0146 a speed at 
-- 9600 baud (bit per sec.)
-----------------------------------------------------------------------------
  baud_timer: process(clk)
  begin 
    if clk'event and clk='1' then	     -- Wait for a clock event
      if baud_count = TBS_uart then     -- Compare the baud count with the TBS uart 
           baud_count <= X"0000";		-- If this is true will the baud count be set 
         en_16_x_baud <= '1';		     -- to zero and enable en_16_x_baud with a one. 
       else
           baud_count <= baud_count + 1; -- If this not are true the counter will 
         en_16_x_baud <= '0';			 -- add one and the en_16_x_baud will be 
      end if;						 -- disable.
    end if;
  end process baud_timer;
-----------------------------------------------------------------------------
--  TIMER COUNTS SYSTEM:
--  This system controls the timer and set the timer flag when the timer is 
--  run out. 
-----------------------------------------------------------------------------
  Timer0_control: process(clk)
  begin 
    if(clk'event and clk='1' and TR0='1') then -- Wait for a clock event and 
      if Timer0 = TC0 then 				  -- TR0 is set active to run
         Timer0 <= X"00";		-- Compare the Timer 0 with the TC0 value
	    TF0 <= '1';			-- If this is true will the timer count be set
      else					-- to zero and enable the Timer flag with a one.
         Timer0 <= Timer0 + 1;     -- If this not are true the counter will
	    TF0 <= '0';			-- add one and the Timer flag will be
      end if;					-- disable.
    end if;
  end process Timer0_control;

  Timer1_control: process(clk)
  begin 
    if(clk'event and clk='1' and TR1='1') then -- Wait for a clock event and
      if Timer1 = TC1 then				  -- TR1 is set active to run
         Timer1 <= X"0000";  		-- Compare the Timer 1 with the TC1 value
	    TF1 <= '1';			-- If this is true will the timer count be set
      else					-- to zero and enable the Timer flag with a one.
         Timer1 <= Timer1 + 1;	-- If this not are true the counter will
	    TF1 <= '0';			-- add one and the Timer flag will be
      end if;					-- disable.
    end if;
  end process Timer1_control;

end connectivity;
------------------------------------------------------------------------------------
--
-- END OF FILE EMBEDDED.VHD
--
------------------------------------------------------------------------------------

